// Adapted and modified from Volker Quaschning's sunpos.js
// Global variables:
var SunAzimuth, SunHeight;

function trunc(x)
{ return(Math.round(x - 0.5));
}

function atan2(y, x)
{ var PI2 = 1.5707963267949;
  var temp;

  if (x == 0)
  { if (y>0) {return (PI2);}
      else {if (y<0) {return(-PI2);} else {return(0);}}
  }
  else
  { temp = Math.atan (y/x)
    if (x>0){ return(temp) }
      else {if (y>=0) {return(temp + Math.PI)} else { return(temp - Math.PI)}}
  }
}//atan2

function rad(grad)
{ return(grad * Math.PI / 180);
}

function grad(rad)
{ return(rad * 180 / Math.PI);
}

function DayOfYear(year, month, day)
{ var x;
  var monthday = new
  Array(0,31,59,90,120,151,181,212,243,273,304,334);

  x = monthday[month-1] + day;
  if ((year%4 == 0) && ((year%100!=0)||(year%400==0)) && (month>2)) x++;
  return(x);
}//DayOfYear

function TimeToStr(hour, min)
{ var s = ""
  if (min>59) {hour+=1; min=0}
  if (hour>24) {hour-=24}
  //if (hour<10) s="0"
  s = s + hour+":"
  if (min<10) s+="0"
  s = s + min
  return(s);
}//TimeToStr

function SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone)
{ var J, J2;
  var Zgl, MOZ, WOZ, w;
  var decl;
  var sunaz, sunhi;
  var asinGs;
  var acosAs;

  J2 = 365;
  if (year % 4 == 0) J2++;
  J = DayOfYear(year, month, day);
  MOZ = hour + 1.0/60*min + 1.0/3600*sec - timezone + 1;
  MOZ = MOZ - 4 * (15 - lon) / 60;
  J = J * 360 / J2 + MOZ / 24;
  decl = 0.3948 - 23.2559 * Math.cos(rad(J + 9.1)) - 0.3915 * Math.cos(rad(2 * J + 5.4)) - 0.1764 * Math.cos(rad(3 * J + 26.0));
  Zgl = 0.0066 + 7.3525 * Math.cos(rad(J + 85.9)) + 9.9359 * Math.cos(rad(2 * J + 108.9)) + 0.3387 * Math.cos(rad(3 * J + 105.2));
  WOZ = MOZ + Zgl / 60;
  w = (12 - WOZ) * 15;
  asinGs = Math.cos(rad(w)) * Math.cos(rad(lat)) * Math.cos(rad(decl)) + Math.sin(rad(lat)) * Math.sin(rad(decl));
  if (asinGs > 1) asinGs = 1;
  if (asinGs < -1) asinGs = -1;
  sunhi = grad(Math.asin(asinGs))
  acosAs = (Math.sin(rad(sunhi)) * Math.sin(rad(lat)) - Math.sin(rad(decl))) / (Math.cos(rad(sunhi)) * Math.cos(rad(lat)));
  if (acosAs > 1) acosAs = 1;
  if (acosAs < -1) acosAs = -1;
  sunaz = grad(Math.acos(acosAs));
  if ((WOZ > 12) || (WOZ < 0)) {sunaz = 180 + sunaz} else {sunaz = 180 - sunaz};
  SunAzimuth = Math.round(sunaz*1000)/1000;
  SunHeight = Math.round(sunhi*1000)/1000;
}//SunPositionDIN

function SunRise(year, month, day, lat, lon, timezone)
{ var DeltaT = 900; // 15 Min
  var TD1 = 0 - DeltaT;
  var TD3 = 86400;
  var TD2;
  var hour, min, sec;

  do // Erste halbe Stunde mit Sonnenhhe < 0 finden
  { TD1 = TD1 + DeltaT;
    hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
    SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
  }
  while ((SunHeight>0) && (TD1 > TD3));

  TD2 = TD1 - DeltaT;
  do // Erste halbe Stunde mit Sonnenhhe > 0 finden *)
  { TD2 = TD2 + DeltaT;
    hour = trunc(TD2/3600); min = trunc((TD2%3600)/60); sec = TD2%60;
    SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
  }
  while (((SunHeight < 0) || (TD2<TD1)) && (TD2<TD3));

  if ((TD1 <= TD3) && (TD2 <= TD3))
  { DeltaT = TD2 - TD1;
    TD1 = TD1 + DeltaT / 2;
    while (DeltaT > 1)
    { DeltaT = DeltaT / 2;
      hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
    SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
      if (SunHeight> 0) {TD1 = TD1 - DeltaT;} else {TD1 = TD1 + DeltaT;}
    }
    hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
    return(TimeToStr(hour,min));
  }
  else return ("");
}//SunRise

function SunSet(year, month, day, lat, lon, timezone)
{ var DeltaT = 900; // 15 Min
  var TD1;
  var TD3 = 0;
  var TD2 = 86400 + DeltaT;
  var hour, min, sec;

  do // letzte halbe Stunde mit Sonnenhhe < 0 finden
  { TD2 = TD2 - DeltaT;
    hour = trunc(TD2/3600); min = trunc((TD2%3600)/60); sec = TD2%60;
    SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
  }
  while ((SunHeight>0) && (TD2 > TD3));

  TD1 = TD2 + DeltaT;
  do // letzte halbe Stunde mit Sonnenhhe > 0 finden
  { TD1 = TD1 - DeltaT;
    hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
    SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
  }
  while (((SunHeight < 0) || (TD1>TD2)) && (TD1>TD3));

  if ((TD1 >= TD3) && (TD2 >= TD3))
  { DeltaT = TD2 - TD1;
    TD1 = TD1 + DeltaT / 2;
    while (DeltaT > 1)
    { DeltaT = DeltaT / 2;
      hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
      SunPositionDIN(year, month, day, hour, min, sec, lat, lon, timezone);
      if (SunHeight> 0) {TD1 = TD1 + DeltaT;} else {TD1 = TD1 - DeltaT;}
    }
    hour = trunc(TD1/3600); min = trunc((TD1%3600)/60); sec = TD1%60;
    return(TimeToStr(hour,min));
  }
  else return ("");
}//SunSet
